// Licensed under the Apache License, Version 2.0
// <LICENSE-APACHE or http://www.apache.org/licenses/LICENSE-2.0> or the MIT license
// <LICENSE-MIT or http://opensource.org/licenses/MIT>, at your option.
// All files in the project carrying such notice may not be copied, modified, or distributed
// except according to those terms.
//! MPR (Multiple Provider Router) definitions
// #include <lmcons.h>
pub const MAX_INTERFACE_NAME_LEN: usize = 256;
pub const MAX_TRANSPORT_NAME_LEN: usize = 40;
pub const MAX_MEDIA_NAME: usize = 16;
pub const MAX_PORT_NAME: usize = 16;
pub const MAX_DEVICE_NAME: usize = 128;
pub const MAX_PHONE_NUMBER_LEN: usize = 128;
pub const MAX_DEVICETYPE_NAME: usize = 16;
