// Feel free to edit this file, it won't be regenerated by gir generator unless removed.

#define PANGO_ENABLE_BACKEND
#define PANGO_ENABLE_ENGINE
#include <pango/pango.h>
#include <pango/pango-modules.h>
