// This file is @generated by no_atomic.sh.
// It is not intended for manual editing.

const NO_ATOMIC: &[&str] = &[
    "bpfeb-unknown-none",
    "bpfel-unknown-none",
    "mipsel-sony-psx",
    "msp430-none-elf",
    "riscv32i-unknown-none-elf",
    "riscv32im-unknown-none-elf",
    "riscv32imc-unknown-none-elf",
];
