/// List of custom properties defined in a declaration list
pub struct VariableDefinitions {
    defs: Vec<VariableDefinition>,
}

pub struct VariableDefinition {
    /// Just the name without the '--' prefix
    name: String,
    value: String,
}

